# fusa-gcc-plugin

This plugin checks that only verified functions are being used during compilation.

## Try it out

Use the `run_container` script to enter a pre canned CentOS Stream 9 environment to try the plugin out, note that the first time
the command is invoked it'll build the necessary image first.

## Usage

```
FuSa Toolchain

This plugin verifies that only FuSa certified functions are used during compilation.

Arguments:

  -fplugin-arg-fusa-dump-apis          - Dump function database to the specified file
  -fplugin-arg-fusa-database-file      - Use function database from another location
  -fplugin-arg-fusa-target-profile     - Target profile for safety requirements. Valid options: asil-b, qm. Default: asil-b
  -fplugin-arg-_fusa-usage             - Print usage and exit
```

You can also check the `Makefile` for inspiration.

## How it works

When this plugin is enabled it captures all function imports and references from distribution provided header files
and checks that only functions conforming to target profile are used. The database of verified function and respective
profiles is by default located in `fusa_verified_toolchain.json`.

### How it really works

The plugin executes in the CFG (Control-Flow Graph) phase at which point we can use the GIMPLE representation 
of all functions to dig out all AST nodes of type `FUNCTION_DECL` which are then inspected further. For each
`FUNCTION_DECL` we look at from which file it originates, SHA256 digest of the file, the line at which it's defined
and the AST hash.

During compilation these variables are compared against the data stored in the database file, any discrepancy is reported
as compilation error at the respective reference or invocation site. 

## generate-database tool

This tool is used to generate JSON database that can be used by the plugin, the input to the tool is the output of the validation suite:

```
./generate-database.py validation/ivos-9/ valid_apis.json
```

## Todo

- Actually implement profiles
  - Database support
  - `fusa_check` tweaks
- AST Hash behaves wonky for C++ due to inlining
